
#pragma used+
sfrb TWBR=0;
sfrb TWSR=1;
sfrb TWAR=2;
sfrb TWDR=3;
sfrb ADCL=4;
sfrb ADCH=5;
sfrw ADCW=4;      
sfrb ADCSRA=6;
sfrb ADMUX=7;
sfrb ACSR=8;
sfrb UBRRL=9;
sfrb UCSRB=0xa;
sfrb UCSRA=0xb;
sfrb UDR=0xc;
sfrb SPCR=0xd;
sfrb SPSR=0xe;
sfrb SPDR=0xf;
sfrb PIND=0x10;
sfrb DDRD=0x11;
sfrb PORTD=0x12;
sfrb PINC=0x13;
sfrb DDRC=0x14;
sfrb PORTC=0x15;
sfrb PINB=0x16;
sfrb DDRB=0x17;
sfrb PORTB=0x18;
sfrb EECR=0x1c;
sfrb EEDR=0x1d;
sfrb EEARL=0x1e;
sfrb EEARH=0x1f;
sfrw EEAR=0x1e;   
sfrb UBRRH=0x20;
sfrb UCSRC=0X20;
sfrb WDTCR=0x21;
sfrb ASSR=0x22;
sfrb OCR2=0x23;
sfrb TCNT2=0x24;
sfrb TCCR2=0x25;
sfrb ICR1L=0x26;
sfrb ICR1H=0x27;
sfrw ICR1=0x26;   
sfrb OCR1BL=0x28;
sfrb OCR1BH=0x29;
sfrw OCR1B=0x28;  
sfrb OCR1AL=0x2a;
sfrb OCR1AH=0x2b;
sfrw OCR1A=0x2a;  
sfrb TCNT1L=0x2c;
sfrb TCNT1H=0x2d;
sfrw TCNT1=0x2c;  
sfrb TCCR1B=0x2e;
sfrb TCCR1A=0x2f;
sfrb SFIOR=0x30;
sfrb OSCCAL=0x31;
sfrb TCNT0=0x32;
sfrb TCCR0=0x33;
sfrb MCUCSR=0x34;
sfrb MCUCR=0x35;
sfrb TWCR=0x36;
sfrb SPMCR=0x37;
sfrb TIFR=0x38;
sfrb TIMSK=0x39;
sfrb GIFR=0x3a;
sfrb GICR=0x3b;
sfrb SPL=0x3d;
sfrb SPH=0x3e;
sfrb SREG=0x3f;
#pragma used-

#asm
	#ifndef __SLEEP_DEFINED__
	#define __SLEEP_DEFINED__
	.EQU __se_bit=0x80
	.EQU __sm_mask=0x70
	.EQU __sm_powerdown=0x20
	.EQU __sm_powersave=0x30
	.EQU __sm_standby=0x60
	.EQU __sm_ext_standby=0x70
	.EQU __sm_adc_noise_red=0x10
	.SET power_ctrl_reg=mcucr
	#endif
#endasm

#pragma used+

void delay_us(unsigned int n);
void delay_ms(unsigned int n);

#pragma used-

void alarm(const unsigned char repeat);
void lights(const unsigned char enable);

bit engine=0;
bit lights_on=0;
bit force_off=0;
unsigned int measured_voltage[128          ];
unsigned char measure_cycle=0;
unsigned char pre_start=255;

interrupt [2] void ext_int0_isr(void)
{
unsigned char valid_int0=255          ;

if(!(MCUCR&0x03))
{
while(--valid_int0)
{
delay_ms(3);
if(PIND.2      )
return;        
}

if(!lights_on)
force_off=1;

MCUCR|=0x03;
return;
}

if((MCUCR&0x03)==0x03)
{
while(--valid_int0)
{
delay_ms(3);
if(!PIND.2      )
return;        
}
force_off=1;
GICR&=0xBF; 
return;
}
}

interrupt [10] void timer0_ovf_isr(void)
{
if(--pre_start==0)
TCCR0=0;
}

interrupt [9] void timer1_ovf_isr(void)
{
TCCR1B=0;
}

interrupt [5] void timer2_ovf_isr(void)
{
PORTD.4     =~PORTD.4     ;
#asm("cli")
delay_ms(100);
#asm("sei")
}

interrupt [15] void adc_isr(void)
{
unsigned int adc_data;
unsigned char i;
unsigned char voltage_ok=0;
unsigned char voltage_ng=0;

adc_data=ADCW;

measured_voltage[measure_cycle]=(int)(adc_data*2.56*6.5); 

for(i=0;i<128          ;i++)
{
if(measured_voltage[i]>13000  )
voltage_ok++;
else
voltage_ng++;                          
}

if(++measure_cycle==128          )
measure_cycle=0;

if(voltage_ok==128          )
{
engine=1;        
PORTD.4     =0;   
}
if(voltage_ng==128          )
{
engine=0;
PORTD.4     =1;
}
}

void alarm(const unsigned char repeat)
{
unsigned char i;

if(pre_start)
return;

for(i=0;i<repeat;i++)
{

#asm("cli")
PORTD.1     =1;
delay_ms(150);
PORTD.1     =0;
delay_ms(80);
#asm("sei")
}

TCCR1B=0x03;    
while(TCCR1B);     
}

void lights(const unsigned char enable)
{
switch(enable)
{

case 0:
PORTD.5     =0;    
PORTD.6     =0;    
TCCR2=0x07;     
lights_on=0;
break;

case 1:
#asm("cli")
PORTD.6     =1;    
delay_ms(1000); 
PORTD.5     =1;    
PORTD.1     =1;
delay_ms(700);
PORTD.1     =0;
lights_on=1;
#asm("sei")
break;
}
}

void main(void)
{
unsigned char j;

PORTB=0x00;
DDRB=0x00;

PORTC=0x00;
DDRC=0x00;

PORTD=0x10;
DDRD=0x72;

TCCR0=0x05;
TCNT0=0x00;

TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

ASSR=0x00;
TCCR2=0x00;
TCNT2=0x00;
OCR2=0x00;

GICR|=0x40;
MCUCR=0x00;
GIFR=0x40;

TIMSK=0x45;

ACSR=0x80;
SFIOR=0x00;

ADMUX=0xC0 | 0x04;
ADCSRA=0xEF;

for(j=0;j<5;j++)
{
PORTD.4     =0;
delay_ms(80);
PORTD.4     =1;
delay_ms(150);
}

#asm("sei")

while (1)
{

if(force_off)
{

lights(0);
}

else
{      

if(engine && !lights_on)
lights(1);

if(lights_on && !engine) 
alarm(3);

if(!lights_on && !engine)
alarm(5);
}        
};
}
